@@ -35,6 +35,8 @@ class GoogleReader(object):
         self.specialFeeds   = {}
         self.orphanFeeds    = []
         self.userId         = None
+        self.addTagBacklog  = {}
+        self.inItemTagTransaction   = False
 
     def toJSON(self):
         """
@@ -131,7 +133,7 @@ class GoogleReader(object):
 
         return True
 
-    def _getFeedContent(self, url, excludeRead=False, continuation=None):
+    def _getFeedContent(self, url, excludeRead=False, continuation=None, loadLimit=20, since=None, until=None):
         """
         A list of items (from a feed, a category or from URLs made with SPECIAL_ITEMS_URL)
 
@@ -151,6 +153,11 @@ class GoogleReader(object):
             parameters['xt'] = 'user/-/state/com.google/read'
         if continuation:
             parameters['c'] = continuation
+        parameters['n'] = loadLimit
+        if since:
+            parameters['ot'] = since
+        if until:
+            parameters['nt'] = until
         contentJson = self.httpGet(url, parameters)
         return json.loads(contentJson, strict=False)
 
@@ -160,26 +167,65 @@ class GoogleReader(object):
             objects.append(Item(self, item, parent))
         return objects
 
-    def getFeedContent(self, feed, excludeRead=False, continuation=None):
+    def getFeedContent(self, feed, excludeRead=False, continuation=None, loadLimit=20, since=None, until=None):
         """
         Return items for a particular feed
         """
-        return self._getFeedContent(feed.fetchUrl, excludeRead, continuation)
+        return self._getFeedContent(feed.fetchUrl, excludeRead, continuation, loadLimit, since, until)
 
-    def getCategoryContent(self, category, excludeRead=False, continuation=None):
+    def getCategoryContent(self, category, excludeRead=False, continuation=None, loadLimit=20, since=None, until=None):
         """
         Return items for a particular category
         """
-        return self._getFeedContent(category.fetchUrl, excludeRead, continuation)
+        return self._getFeedContent(category.fetchUrl, excludeRead, continuation, loadLimit, since, until)
+
+    def _modifyItemTag(self, item_id, action, tag):
+        """ wrapper around actual HTTP POST string for modify tags """
+        return self.httpPost(ReaderUrl.EDIT_TAG_URL,
+                             {'i': item_id, action: tag, 'ac': 'edit-tags'})
 
     def removeItemTag(self, item, tag):
-        return self.httpPost(ReaderUrl.EDIT_TAG_URL,
-                             {'i': item.id, 'r': tag, 'ac': 'edit-tags', })
+        """
+        Remove a tag to an individal item.
+
+        tag string must be in form "user/-/label/[tag]"
+        """
+        return self._modifyItemTag(item.id, 'r', tag)
+
+    def beginAddItemTagTransaction(self):
+        if self.inItemTagTransaction:
+            raise Exception("Already in addItemTag transaction")
+        self.addTagBacklog = {}
+        self.inItemTagTransaction = True
 
     def addItemTag(self, item, tag):
-        return self.httpPost(
-            ReaderUrl.EDIT_TAG_URL,
-            {'i': item.id, 'a': tag, 'ac': 'edit-tags', })
+        """
+        Add a tag to an individal item.
+
+        tag string must be in form "user/-/label/[tag]"
+        """
+        if self.inItemTagTransaction:
+            # XXX: what if item's parent is not a feed?
+            if not tag in self.addTagBacklog:
+                self.addTagBacklog[tag] = []                
+            self.addTagBacklog[tag].append({'i': item.id, 's': item.parent.id})
+            return "OK"
+        else:
+            return self._modifyItemTag(item.id, 'a', tag)
+
+    
+    def commitAddItemTagTransaction(self):
+        if self.inItemTagTransaction:
+            for tag in self.addTagBacklog:
+                itemIds = [item['i'] for item in self.addTagBacklog[tag]]
+                feedIds = [item['s'] for item in self.addTagBacklog[tag]]
+                self.httpPost(ReaderUrl.EDIT_TAG_URL,
+                    {'i': itemIds, 'a': tag, 'ac': 'edit-tags', 's': feedIds})
+            self.addTagBacklog = {}
+            self.inItemTagTransaction = False
+            return True
+        else:
+            raise Exception("Not in addItemTag transaction")
 
     def markFeedAsRead(self, feed):
         return self.httpPost(
